import Vue from 'vue'
import Router from 'vue-router'

import userManagement from './moudules/userManage'
import systemManage from './moudules/systemManage'
import enterpriseManage from './moudules/enterpriseManage'
import capitalManage from './moudules/capitalManage'
import storeManage from './moudules/storeManage'
import monitorManage from './moudules/monitorManage'
import cyAccountManage from './moudules/cyAccountManage'
import creditManage from './moudules/creditManage'

Vue.use(Router)

/* Layout */
import Layout from '@/layout'

/**
 * constantRoutes
 * a base page that does not have permission requirements
 * all roles can be accessed
 */
export const constantRoutes = [
  {
    path: '/login',
    component: () => import('@/views/login/index'),
    hidden: true
  },
  {
    path: '/perview',
    component: () => import('@/views/perview/index'),
    hidden: true
  },

  {
    path: '/404',
    component: () => import('@/views/404'),
    hidden: true
  },

  {
    path: '/',
    component: Layout,
    redirect: '/dashboard',
    name: 'DashboardIndex',
    children: [{
      path: 'dashboard',
      name: 'Dashboard',
      component: () => import('@/views/dashboard/index'),
      meta: { title: '首页', icon: 'home' }
    }]
  },
  storeManage,
  enterpriseManage,
  creditManage,
  capitalManage,
  userManagement,
  monitorManage,
  cyAccountManage,
  systemManage,

  // 404 page must be placed at the end !!!
  { path: '*', redirect: '/404', hidden: true }
]

/**
 * asyncRoutes
 * the routes that need to be dynamically loaded based on user roles
 */
export const asyncRoutes = [
  userManagement,
  systemManage
]
const createRouter = () => new Router({
  // mode: 'history', // require service support
  base: '/factoring-trade',
  scrollBehavior: () => ({ y: 0 }),
  routes: constantRoutes
})

const router = createRouter()

// Detail see: https://github.com/vuejs/vue-router/issues/1234#issuecomment-357941465
export function resetRouter() {
  const newRouter = createRouter()
  router.matcher = newRouter.matcher // reset router
}

export default router
